/* 	ConnectionReader.java
 *	Copyright (c) 2004-2006, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Feb 11, 2004
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]       [Who]                   [What]
 * 14-01-2006   dennis@brains2b.nl      Use new Saxo 0.20 parser
 * 09-02-2007   dennis@brains2b.nl      Use dynamic classpath
 * 14-03-2007   dennis@brains2b.nl      Fixed problem reading driver information
 * 11-10-2007	dennis@brains2b.nl		Write Driver content to document correctly
 */
package org.brains2b.jdbc;

import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;

import org.brains2b.saxo.SAXWriter;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.11.1 [14-03-2007]
 */
public class ConnectionDocument implements ContentHandler, Runnable {
	
	public static final int READ=0;
	public static final int WRITE=1;
	
	private int action=0;
	private String filename;
	
	private Object last;
	private String lastName;
	private Vector drivers;
	private Vector connections;
	
	/**
	 * 
	 */
	public ConnectionDocument() {
		super();
	}

	/* (non-Javadoc)
	 * @see org.xml.sax.DocumentHandler#setDocumentLocator(org.xml.sax.Locator)
	 */
	public void setDocumentLocator(Locator arg0) {

	}

	/* (non-Javadoc)
	 * @see org.xml.sax.DocumentHandler#startDocument()
	 */
	public void startDocument() throws SAXException {
		
		drivers=new Vector();
		connections=new Vector();
	}

	/* (non-Javadoc)
	 * @see org.xml.sax.DocumentHandler#endDocument()
	 */
	public void endDocument() throws SAXException {
		
	}

	/* (non-Javadoc)
	 * @see org.xml.sax.DocumentHandler#startElement(java.lang.String, org.xml.sax.AttributeList)
	 */
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
     	if (localName.equals("driver")) {
			last=new Driver();	
            drivers.add(last);
		} else if (localName.equals("connection")) {
			last=new Connection();
            connections.add(last);
		} else if (localName.equals("classpath")) {
			last=new ArrayList();
		} else {
			lastName=localName;
		}

	}

	/* (non-Javadoc)
	 * @see org.xml.sax.DocumentHandler#endElement(java.lang.String)
	 */
    public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
     	if (localName.equals("classpath") && last instanceof ArrayList && !drivers.isEmpty()) {
			URL[] urls= new URL[((ArrayList) last).size()];
			((ArrayList) last).toArray(urls);
            Driver d= (Driver) drivers.lastElement();
			d.setClassPath(urls);
            last=d;
		}

	}

	/* (non-Javadoc)
	 * @see org.xml.sax.DocumentHandler#characters(char[], int, int)
	 */
	public void characters(char[] arg0, int arg1, int arg2)
		throws SAXException {
		if (last instanceof Driver) {
			Driver d= (Driver) last;
			if (lastName.equals("name")) {
				d.setClassName(new String(arg0));
			} else if (lastName.equals("url")) {
				d.setUrlprefix(new String(arg0));
			}
		} else if (last instanceof Connection) {
			Connection c= (Connection) last;
			if (lastName.equals("driverName")) {
				c.setDriver(new String(arg0));
			} else if (lastName.equals("url")) {
				c.setUrl(new String(arg0));
			} else if (lastName.equals("user")) {
				c.setUser(new String(arg0));
			} else if (lastName.equals("password")) {
				c.setPassword(new String(arg0));
			}
		} else if (last instanceof ArrayList) {
			try {
				((ArrayList) last).add(new URL(new String(arg0)));
			} catch (MalformedURLException muex) {
				throw new SAXException(muex);
			}
		}

	}

	/* (non-Javadoc)
	 * @see org.xml.sax.DocumentHandler#ignorableWhitespace(char[], int, int)
	 */
	public void ignorableWhitespace(char[] arg0, int arg1, int arg2)
		throws SAXException {

	}

	/* (non-Javadoc)
	 * @see org.xml.sax.DocumentHandler#processingInstruction(java.lang.String, java.lang.String)
	 */
	public void processingInstruction(String arg0, String arg1)
		throws SAXException {

	}

	/**
	 * @return
	 */
	public Vector getConnections() {
		return connections;
	}

	/**
	 * @return
	 */
	public Vector getDrivers() {
		return drivers;
	}

	/**
	 * @param vector
	 */
	public void setConnections(Vector vector) {
		connections = vector;
	}

	/**
	 * @param vector
	 */
	public void setDrivers(Vector vector) {
		drivers = vector;
	}

	public void writeDocument(String fileName) throws IOException, SAXException {
		FileWriter fw=new FileWriter(fileName);
		SAXWriter sw=new SAXWriter(fw);
		sw.startDocument();
		sw.startElement("connectionInfo",null);
		if (drivers!=null) {
			Iterator it=drivers.iterator();
			while(it.hasNext()) {
				write(sw,(Driver) it.next());
			}
		} 
		if (connections!=null) {
			Iterator it=connections.iterator();
			while(it.hasNext()) {
				write(sw,(Connection) it.next());
			}
		}
		sw.endElement("connectionInfo");
		fw.close();
	}
	
	
	private void write(SAXWriter w,Driver d) throws SAXException {
		w.startElement("driver",null);
		w.startElement("name",null);
		w.setContent(d.getClassName());
		w.endElement("name");
		w.startElement("url",null);
		w.setContent(d.getUrlprefix());
		w.endElement("url");
		if (d.getClassPath()!=null) {
			w.startElement("classpath",null);
			for (int i=0;i<d.getClassPath().length;i++) {
				w.startElement("path",null);
				w.setContent(d.getClassPath()[i].toString());
				w.endElement("path");
			}
			w.endElement("classpath");
		}
		w.endElement("driver");
	}
	
	private void write (SAXWriter w, Connection c) throws SAXException {
		w.startElement("connection",null);
		w.startElement("driverName",null);
		w.setContent(c.getDriver());
		w.endElement("driverName");
		w.startElement("url",null);
		w.setContent(c.getUrl());
		w.endElement("url");
		w.startElement("user",null);
		w.setContent(c.getUser());
		w.endElement("user");
		w.startElement("password",null);
		w.setContent(c.getPassword());
		w.endElement("password");
		w.endElement("connection");
	}
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		if (getAction()==WRITE) {
			try {
				writeDocument(getFilename());
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}
	}

	/**
	 * @return
	 */
	public int getAction() {
		return action;
	}

	/**
	 * @return
	 */
	public String getFilename() {
		return filename;
	}

	/**
	 * @param i
	 */
	public void setAction(int i) {
		action = i;
	}

	/**
	 * @param string
	 */
	public void setFilename(String string) {
		filename = string;
	}

	public void addDriver(Driver d) {
		if (drivers==null) {
			drivers=new Vector();
		}
		drivers.add(d);
	}
	
	public void removeDriver(Driver d) {
		if (drivers==null) {
			return;
		}
		drivers.remove(d);
	}
	
	public void addConnection(Connection c) {
		if (connections==null) {
			connections=new Vector();
		}
		connections.add(c);
	}
	
	public void removeConnection(Connection c) {
		if (connections!=null) {
			connections.remove(c);
		}
	}

    public void endPrefixMapping(String prefix) throws SAXException {
        
    }

    public void skippedEntity(String name) throws SAXException {
        
    }

    public void startPrefixMapping(String prefix, String uri) throws SAXException {
        
    }
}
