/* 	ConnectionView.java
 *	Copyright (c) 2004-2006, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Feb 11, 2004
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Jun 3, 2005	dennis@brains2b.nl		Select the correct driver in edit from table, 
 * 										Fixed problem with column width, add, delete and test
 * 14-01-2006   dennis@brains2b.nl      Use new Saxo 0.20 parser
 * 09-02-2007   dennis@brains2b.nl      Use new LocalConnector, use dynamic classpath
 * 15-03-2007   dennis@brains2b.nl      Fixed problem with message, open connection on double click
 * 11-10-2007	dennis@brains2b.nl		Prevent same drivers being added to document
 */
package org.brains2b.swingext.connect;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import org.brains2b.jdbc.Connection;
import org.brains2b.jdbc.ConnectionDocument;
import org.brains2b.jdbc.Driver;
import org.brains2b.saxo.SAXReader;
import org.brains2b.sql.LocalConnector;
import org.brains2b.swingext.form.Form;
import org.xml.sax.SAXException;



/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.11.2 [15-03-2007]
 */
public class ConnectionView extends JPanel {

	private class FocusHandler implements FocusListener {

		/* (non-Javadoc)
		* @see java.awt.event.FocusListener#focusGained(java.awt.event.FocusEvent)
		*/
		public void focusGained(FocusEvent arg0) {
			//Not implemented

		}

		/* (non-Javadoc)
		 * @see java.awt.event.FocusListener#focusLost(java.awt.event.FocusEvent)
		 */
		public void focusLost(FocusEvent arg0) {
		    
			message.setText(" ");
			Connection c= getCurrentConnection();
			int row = table.getSelectedRow();
			String name = arg0.getComponent().getName();
			String value = null;
			if (name.equals("driverName")) {
				Driver d =
					(Driver) ((JComboBox) arg0.getComponent())
						.getSelectedItem();
				if (d==null) {
					return;
				}	
				value = d.getClassName(); 
				c.setDriver(value);
				if (isEmpty(url.getText())) {
					url.setText(d.getUrlprefix());
					c.setUrl(d.getUrlprefix());
				}
			} else if (name.equals("password")) {
				value =
					new String(
						((JPasswordField) arg0.getComponent()).getPassword());
				c.setPassword(value);
			} else if (arg0.getComponent() instanceof JTextField) {
				value = ((JTextField) arg0.getComponent()).getText();
				if (name.equals("user")) {
					c.setUser(value);
				} else if (name.equals("url")) {
					c.setUrl(value);
				}
			}
			
			btnTest.setEnabled(isConnectionValid(c,false));
			
			((ConnectionTableModel) table.getModel()).fireTableRowsUpdated(
				row,
				row + 1);

		}

	}

	private class ActionHandler implements ActionListener {

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent arg0) {
			if (arg0.getActionCommand().equals("driver.Add")) {
				getCurrentConnection();
				fireDriver(null);
			} else if (arg0.getActionCommand().equals("driver.Edit")) {
				getCurrentConnection();
				fireDriver((Driver) drivers.getSelectedItem());
			} else if (arg0.getActionCommand().equals("new")) {
			    int idx=table.getSelectedRow();
			    if (idx==-1 || isConnectionValid((Connection) document.getConnections().get(idx),false)) {
					table.clearSelection();
					document.addConnection(new Connection());
					idx = document.getConnections().size() - 1;
			    }
				if (drivers.getItemCount()>0) {
					drivers.setSelectedIndex(0);
				}
				url.setText("");
				user.setText("");
				password.setText("");
				((ConnectionTableModel) table
						.getModel())
						.fireTableRowsInserted(
					idx,
					idx);
				
				table.addRowSelectionInterval(idx, idx);
			} else if (arg0.getActionCommand().equals("remove")) {
				int idx = table.getSelectedRow();
				if (idx != -1) {
					Connection c =
						(Connection) document.getConnections().get(idx);
					document.removeConnection(c);
					(
						(ConnectionTableModel) table
							.getModel())
							.fireTableRowsDeleted(
						idx,
						idx);
					
					if (idx>0) {idx+=-1;}
					table.getSelectionModel().setSelectionInterval(idx,idx);
				}

			} else if (arg0.getActionCommand().equals("test")) {
				int idx = table.getSelectedRow();
				if (idx != -1) {
					Connection c =
						(Connection) document.getConnections().get(idx);
				    String classpath = findClassPath(c.getDriver());		
					LocalConnector con=new LocalConnector(
						classpath,
                        c.getDriver(),
						c.getUrl(),
						c.getUser(),
						c.getPassword());
					try {
						con.getConnection();
						setMessage("Succes!", Color.blue);
					} catch (SQLException sqex) {
						setMessage(sqex.getMessage(),Color.red);
					}
				}
			}
		}

	}

	private class SelectionHander implements ListSelectionListener {

		/* (non-Javadoc)
		* @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
		*/
		public void valueChanged(ListSelectionEvent arg0) {
		    
			int row = table.getSelectedRow();
			boolean b=false;
			if (row!=-1 && row<table.getRowCount()) {
				
				Connection c = (Connection) document.getConnections().get(row);
				b=true;
				for (int i=0;i<drivers.getItemCount();i++) {
				    Driver driver= (Driver) drivers.getItemAt(i);
				    if (c.getDriver()!=null && c.getDriver().equals(driver.getClassName())) {
				        drivers.setSelectedIndex(i);
				    }
				}
				
				url.setText(c.getUrl());
				user.setText(c.getUser());
				password.setText(c.getPassword());
			}
			
			btnDelete.setEnabled(b);
			btnTest.setEnabled(b);
		}
		
		

	}

	private class ConnectionTableModel extends DefaultTableModel {

		public ConnectionTableModel() {
			super();
		}

		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getColumnClass(int)
		 */
		public Class getColumnClass(int arg0) {
			return String.class;
		}

		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getRowCount()
		 */
		public int getRowCount() {
			if (document != null && document.getConnections() != null) {
				return document.getConnections().size();
			}
			return 0;
		}

		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getValueAt(int, int)
		 */
		public Object getValueAt(int arg0, int arg1) {
			if (document == null || document.getConnections() == null) {
				return null;
			}
			Connection c = (Connection) document.getConnections().get(arg0);
			switch (arg1) {
				case 0 :
					return c.getDriver();
				case 1 :
					return c.getUrl();
				case 2 :
					return c.getUser();
			}
			return null;
		}

		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#isCellEditable(int, int)
		 */
		public boolean isCellEditable(int arg0, int arg1) {
			return false;
		}

		

	}

    class MouseHandler extends MouseAdapter {
        
        public void mouseClicked(MouseEvent e) {
            if (e.getClickCount()>=2) {
                Container c= getParent();
                if (c instanceof Form) {
                    ((Form) c).performOk();
                }
            }
        }
        
    }
    
	private static ConnectionDocument document;
	private JTextField message;
	private JComboBox drivers;
	private JTextField url;
	private JTextField user;
	private JTextField password;
	private JTable table;
	private JButton btnDelete;
	private JButton btnTest;

	public ConnectionView() {
		super();
		init();
		initConnections();
	}

	public boolean close() {
		boolean result=true;
		if (document.getConnections()!=null) {
			for (int i=0;i<document.getConnections().size();i++) {
				result = isConnectionValid((Connection) document.getConnections().get(i),true);
				if (!result) break; 
			}
				
		}
		if (result) writeConnections();
		return result;
	}
	
	public Properties getSelectedConnection() {
		int idx=table.getSelectedRow();
		if (idx!=-1) {
			Connection c=(Connection) document.getConnections().get(idx);
			Properties p=new Properties();
			p.setProperty("driver", c.getDriver() );
			p.setProperty("url", c.getUrl() );
			p.setProperty("user", c.getUser() );
			p.setProperty("password", c.getPassword());
			String path=findClassPath(c.getDriver());
			if (path!=null) {
				p.setProperty("classpath",path);
			}
			return p;
		} 
		return null;
	}

	private void init() {
		ActionHandler ah = new ActionHandler();
		FocusHandler fh = new FocusHandler();
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		JPanel list = new JPanel(new BorderLayout());
		
		table = new JTable();
		table.getSelectionModel().addListSelectionListener(
			new SelectionHander());
        table.addMouseListener(new MouseHandler());
		JScrollPane scrollTable = new JScrollPane();
		scrollTable.setPreferredSize(new Dimension(400, 150));
		scrollTable.setViewportView(table);
		table.setModel(new ConnectionTableModel());
		setColumnModel(table);
		list.add(scrollTable, BorderLayout.CENTER);
		
		
		
		JPanel buttons = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		JButton btnNew = new JButton("New");
		btnNew.setActionCommand("new");
		btnNew.addActionListener(ah);
		buttons.add(btnNew);
		btnDelete = new JButton("Remove");
		btnDelete.setActionCommand("remove");
		btnDelete.addActionListener(ah);
		btnDelete.setEnabled(false);
		buttons.add(btnDelete);
		list.add(buttons, BorderLayout.SOUTH);
		add(list, BorderLayout.CENTER);
		JPanel detail = new JPanel(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.insets = new Insets(5, 5, 5, 5);
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = 0;
		detail.add(new JLabel("driver:"), gbc);
		drivers = new JComboBox();
		drivers.setName("driverName");
		drivers.addFocusListener(fh);
		gbc.gridx++;
		gbc.gridwidth = 2;
		detail.add(drivers, gbc);
		JButton btnDriver = new JButton("edit");
		btnDriver.setActionCommand("driver.Edit");
		btnDriver.addActionListener(ah);
		gbc.gridwidth = 1;
		gbc.gridx += 2;
		detail.add(btnDriver, gbc);
		
		JButton btnAddDriver = new JButton("add");
		btnAddDriver.setActionCommand("driver.Add");
		btnAddDriver.addActionListener(ah);
		gbc.gridwidth = 1;
		gbc.gridx ++;
		detail.add(btnAddDriver, gbc);

		gbc.gridx = 0;
		gbc.gridy++;
		detail.add(new JLabel("url:"), gbc);
		url = new JTextField();
		url.setName("url");
		url.addFocusListener(fh);
		gbc.gridx++;
		gbc.gridwidth = 4;
		detail.add(url, gbc);

		gbc.gridwidth = 1;
		gbc.gridx = 0;
		gbc.gridy++;
		detail.add(new JLabel("user:"), gbc);
		user = new JTextField();
		user.setName("user");
		user.addFocusListener(fh);
		gbc.gridx++;
		gbc.weightx = 0.40d;
		detail.add(user, gbc);

		gbc.gridx++;
		gbc.weightx = 0d;
		detail.add(new JLabel("password:"), gbc);
		gbc.gridx++;
		gbc.gridwidth = 2;
		gbc.weightx = 0.40d;
		password = new JPasswordField();
		password.setName("password");
		password.addFocusListener(fh);
		detail.add(password, gbc);
		
		
		message=new JTextField(" ");
        message.setOpaque(false);
        message.setEditable(false);
        message.setFocusable(false);
        message.setBorder(BorderFactory.createEmptyBorder());
        message.setForeground(Color.red);
		gbc.gridx=0;
		gbc.gridy++;
        gbc.gridwidth=3;
		detail.add(message,gbc);
		btnTest=new JButton("test");
		btnTest.setActionCommand("test");
		btnTest.addActionListener(ah);
		btnTest.setEnabled(false);
		gbc.gridx+=3;
		detail.add(btnTest,gbc);
		add(detail, BorderLayout.SOUTH);
	}

	private void setColumnModel(JTable table) {
		int idx = 0;
		DefaultTableColumnModel model=new DefaultTableColumnModel();
		TableColumn t = new TableColumn(idx++,100);
		t.setIdentifier("driverName");
		t.setHeaderValue("driver");
		//t.setWidth(100);
		model.addColumn(t);
		t = new TableColumn(idx++,125);
		t.setIdentifier("url");
		t.setHeaderValue("url");
		//t.setWidth(125);
		model.addColumn(t);
		t = new TableColumn(idx++,50);
		t.setIdentifier("user");
		t.setHeaderValue("user");
		//t.setWidth(50);
		model.addColumn(t);
		table.setColumnModel(model);
	}

	private void initConnections() {
		if (document == null) {
			try {
				document = new ConnectionDocument();
				SAXReader p = new SAXReader();
				p.setContentHandler(document);
				p.parse("conf/connections.xml");

				fillDrivers();

			} catch (SAXException saex) {

			} catch (FileNotFoundException fnex) {
				// no recent file; Make one later
			} catch (IOException iex) {
			}
		}

	}

	private void writeConnections() {
		document.setAction(ConnectionDocument.WRITE);
		document.setFilename("conf/connections.xml");
		Thread t = new Thread(document); // fire and forget
		t.start();
	}

	private void fillDrivers() {
		if (document.getDrivers() != null) {
			Iterator it = document.getDrivers().iterator();
			while (it.hasNext()) {
				drivers.addItem(it.next());
			}
		}
	}
	
	private void setMessage(String text, Color c) {
		message.setText(text);
        message.setForeground(c);
        message.setCaretPosition(0);
	}
	
	private boolean isConnectionValid(Connection c, boolean showMessage) {
		boolean result=true;
		int idx=document.getConnections().indexOf(c);
		if (isEmpty(c.getDriver())) {
			 if (showMessage) setMessage("Did not define a driver for this connection",Color.red);
			 result=false;
		} else if (isEmpty(c.getUrl())) {
			if (showMessage) setMessage("Did not define a URL for this connection", Color.red);
			result=false;
		}
		if (!result) {
			table.clearSelection();
			table.addRowSelectionInterval(idx,idx);
		}
		return result;
	}
	
	private final static boolean isEmpty(String s) {
		if (s==null|| s.trim().length()==0) {
			return true; 
		} 
		return false;
	}
	
	private void fireDriver(Driver driver) {
		Driver d =
			DriverView.showDailog(
				    SwingUtilities.windowForComponent(
					ConnectionView.this),
				document.getDrivers(),driver);
		if (d != null) {
			if ("DELETED".equals(d.getClassName())) {
				document.removeDriver(d);
				drivers.removeItem(d);
				drivers.requestFocus();
			} else {
				document.getDrivers().remove(d);
				document.addDriver(d);
				drivers.addItem(d);
				drivers.setSelectedItem(d);
				drivers.requestFocus();
			}
		}
	}
	
	private Connection getCurrentConnection() {
		int row = table.getSelectedRow();
		Connection c=null;
		if (row == -1) {
			c=new Connection();
			document.addConnection(c);
			int newRow=document.getConnections().size()-1;
			((DefaultTableModel) table.getModel()).fireTableRowsInserted(newRow,newRow);
			table.addRowSelectionInterval(newRow,newRow);
		} else {
			c = (Connection) document.getConnections().get(row);
		}
		return c;
	}
	
	private String findClassPath(String driverName) {
		if (driverName==null) return null;
		Vector drivers=document.getDrivers();
		if (drivers!=null) {
			for (int i=0;i<drivers.size();i++) {
				if (driverName.equals(((Driver) drivers.get(i)).getClassName())) {
					URL[] cs= ((Driver) drivers.get(i)).getClassPath();
					if (cs!=null && cs.length>0) {
						StringBuffer sb=new StringBuffer();
						for (int j=0;j<cs.length;j++) sb.append(cs[j].getPath());
						return sb.toString();
					}
				}
			}
		}
		return null;
	}
}