/* 	AboutDialog.java
 *	Copyright 	(c) 2002-2005, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]			[Who]							[What]
 * 16-02-2003	dennis@brains2b.nl 	Protected against file not found, NullPointerException
 * 								Added javadoc comments
 * 01-08-2005	dennis@brains2b.nl	Added border around icon, set insets
 * TODO Needs to be rewritten to create the actual panel at the backend
 * TODO Should be able to read a Manifest file
 * TODO Q: Can this be moved to FrameworkDialog?
 */
package org.brains2b.swingext.dialog;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.border.BevelBorder;

import org.brains2b.swingext.gui.IconFactory;
import org.brains2b.util.StringHelper;

/**
 * This class opens an standard About dialog which you can fill through your
 * application and will be layed out by the About Dialog itself
 * <p>
 * <UL>
 * The following properties can be set, but are all optional
 * <LI><b>author</b> the name of the person or organisation that created it.
 * <LI><b>date</b> the date of the current version
 * <LI><b>logo</b> the name of the resource file containing the application
 * logo
 * <LI><b>text</b> some text displayed as a read only JEditPane about this
 * application. Will scroll if required.
 * <LI><b>title</b> the name of the application
 * <LI><b>version</b> the current version of the application
 * <LI><b>contributor</b> if you have different contributors possibly with
 * different licenses, you can add the resource file containing the logo and
 * text, possibly their license as String or as resource file.
 * <p>
 * The AboutDialog further acts as a default JDialog
 * 
 * 
 * @author dennis@brains2b.org
 * @version 0.30.1 [01-08-2005]
 */
public class AboutPane extends JPanel {

    /**
     * ActionListener for the OK button
     */
    private class ActionEventHandler implements ActionListener {
        /*
         * @see ActionListener#actionPerformed(ActionEvent)
         */
        public void actionPerformed(ActionEvent e) {
            if (e.getSource() == m_ok) {
                close();
            }
        }

    }

    private GridBagConstraints m_gbc;

    private JLabel m_title = new JLabel();

    private JLabel m_version = new JLabel();

    private JLabel m_date = new JLabel();

    private JLabel m_author = new JLabel();

    private JTextPane m_text = new JTextPane();

    private JLabel m_logo = new JLabel();

    private List m_licLogo = new ArrayList();

    private List m_licText = new ArrayList();

    private JButton m_ok;

    private boolean notInit=true;
    /**
     * Constructor for AboutDialog
     * 
     * @see java.awt.Dialog#Dialog(Frame, boolean)
     */
    public AboutPane() {
        super();
    }

    /**
     * Constructor for AboutDialog.
     * <p>
     * WARNING: This function is not implemented yet. It is however not removed
     * while some old applications still rely on it.
     * 
     * @param prop
     *            java.util.Properties properties containing values for
     *            initializing the About Dialog
     * @see java.awt.Dialog#Dialog(Frame, boolean)
     */
    public AboutPane(Properties prop) {
        super();
    }

    
    
    private void init() {
        
        setBorder(BorderFactory.createEmptyBorder(10, 5, 10, 5));
        setLayout(new GridBagLayout());
        m_ok = new JButton("Ok");
        m_ok.addActionListener(new ActionEventHandler());
        m_ok.setMnemonic('o');
        
        // The nitty Griddie
        m_gbc = new GridBagConstraints();
        m_gbc.insets = new Insets(2, 5, 2, 5);
        m_gbc.anchor = GridBagConstraints.WEST;

        m_text.setEditable(false);
        // m_text.setFont(new Font("sans-serif", Font.PLAIN, 10));
        
        m_text.setForeground(Color.black);
        m_text.setBackground(getBackground());
        m_text.setVisible(true);
        
        layoutItem(m_logo, 0, 0, 1, 4);
        layoutItem(m_title, 1, 0, 2, 1);
        layoutItem(getLabel("version "), 1, 1, 1, 1);
        layoutItem(m_version, 2, 1, 1, 1);
        layoutItem(getLabel("date "), 1, 2, 1, 1);
        layoutItem(m_date, 2, 2, 1, 1);
        layoutItem(getLabel("author "), 1, 3, 1, 1);
        layoutItem(m_author, 2, 3, 1, 1);
        layoutItem(new JPanel(), 0, 4, 4, 1);
        if (!StringHelper.isEmpty(m_text.getText())) {
            JScrollPane scroll = new JScrollPane(m_text);
            scroll.setMinimumSize(new Dimension(400,100));
            scroll.setPreferredSize(new Dimension(400,100));
            layoutItem(scroll, 1, 5, 3, 1);
            m_text.setCaretPosition(0);
        }
        layoutItem(new JPanel(), 0, 6, 4, 1);
        int contr = m_licText.size();
        for (int i = 0; i < contr; i++) {
            layoutItem((JComponent) m_licLogo.get(i), 0, 7 + i, 1, 1);
            layoutItem((JComponent) m_licText.get(i), 1, 7 + i, 3, 1);
        }
        m_gbc.gridx= 3;
        m_gbc.gridy= 7 + contr;
        m_gbc.gridwidth = 1;
        m_gbc.gridheight = 1;
        m_gbc.anchor=GridBagConstraints.EAST;
        m_gbc.insets = new Insets(2, 5, 2, 8);
        add(m_ok,m_gbc);
        notInit = false;
    }

    private void layoutItem(JComponent cmp, int x, int y, int w, int h) {
        m_gbc.gridx = x;
        m_gbc.gridy = y;
        m_gbc.gridwidth = w;
        m_gbc.gridheight = h;
        add(cmp, m_gbc);
    }

    public Dimension getPreferredSize() {
        if (notInit) init();
        return super.getPreferredSize();
    }
    private void close() {
        if (getRootPane().getParent() instanceof JInternalFrame) {
            ((JInternalFrame) getRootPane().getParent()).doDefaultCloseAction();      
        } else {
            SwingUtilities.getWindowAncestor(this).dispose();
        }
    }

    private JLabel getLabel(String text) {
        JLabel lbl = new JLabel(text);
        lbl.setForeground(Color.black);
        // lbl.setFont(new Font("sans-serif", Font.PLAIN, 11));
        lbl.setBackground(getBackground());
        return lbl;
    }

    /**
     * sets the logo by reading the fileName as resource.
     * 
     * @param fileName String,
     *            the name of the file which needs to be in the classpath
     */
    public void setLogo(String fileName) {
        setLogo(fileName, true);
    }

    /**
     * sets the logo by reading the fileName as resource.
     * 
     * @param fileName String,
     *            the name of the file which needs to be in the classpath
     * @param border boolean, true if you want to draw a border around the logo
     */
    public void setLogo(String fileName, boolean border) {
        m_logo = getLabel(null);
        if (border)
            m_logo.setBorder(BorderFactory.createCompoundBorder(BorderFactory
                    .createBevelBorder(BevelBorder.LOWERED), BorderFactory
                    .createEmptyBorder(5, 5, 5, 5)));
        m_logo.setIcon(IconFactory.createIcon(fileName, 64));
    }

    /**
     * Sets the title of the application
     * <p>
     * Also sets the title of the Window including the word About.
     * 
     * @see java.awt.Dialog#setTitle(String)
     */
    public void setTitle(String title) {
        m_title = getLabel(title);
        m_title.setBorder(BorderFactory.createEmptyBorder(5, 0, 5, 0));
        Font f = m_title.getFont().deriveFont(Font.BOLD, 30f);
        m_title.setFont(f);
    }

    /**
     * sets the current version of the application
     * 
     * @param version
     *            String, a freeformat String to describe the version.
     */
    public void setVersion(String version) {
        m_version = getLabel(version);
    }

    /**
     * sets the date of the current application
     * 
     * @param date
     *            String , a freeformat String to describe the date.
     */
    public void setDate(String date) {
        m_date = getLabel(date);
    }

    /**
     * sets the name of the person or organisation that made the application
     * 
     * @param author
     *            String, a freeformat String to describe the author of the
     *            application
     */
    public void setAuthor(String author) {
        m_author = getLabel(author);
    }

    /**
     * sets the text of this about box, can be a license or anything else you
     * want to tell the user. Will be shown as multi-line text, with scrollbars
     * if required.
     * <p>
     * You might prefer setTextfromFile if the text is long.
     * 
     * @param text
     *            String, the text you want displayed in this application.
     */
    public void setText(String text) {
        m_text.setText(text);
    }

    private String getTextfromFile(String fileName) throws IOException {
        try {
            InputStream resource = getClass().getResourceAsStream(fileName);
            if (resource == null) {
                File f = new File(fileName);
                if (f.exists()) {
                    resource = new FileInputStream(f);
                }
            }
            BufferedReader bfr = new BufferedReader(new InputStreamReader(
                    resource));

            StringBuffer sb = new StringBuffer();
            String str = bfr.readLine();
            while (str != null) {
                sb.append(str);
                sb.append('\n');
                str = bfr.readLine();
            }
            bfr.close();
            return sb.toString();
        } catch (NullPointerException npex) {
            throw new FileNotFoundException("File " + fileName
                    + " cannot be found.");
        }

    }

    /**
     * sets the text for this application to be read from a resource file.
     * <p>
     * The file is read and displayed line by line. So be sure you have carriage
     * returns in you text where appropriate, otherwise it might be all garbaled
     * up on screen.
     * 
     * @param fileName
     *            String, the name of the file to be read as resource. This
     *            needs to be found in the classpath of the application.
     * @throws IOException,
     *             thrown if the file cannot be found or accessed.
     */
    public void setTextfromFile(String fileName) throws IOException {

        setText(getTextfromFile(fileName));

    }

    /**
     * adds a Contributor to an aboutbox. You can set the logoFile or the text
     * or both.
     * 
     * @param logoFile
     *            String, the name of the file to be read as resource. This
     *            needs to be found in the classpath of the application.
     * @param text
     *            String, a possible multi-line text with text possibly the
     *            license of the contributor
     */
    public void addContributor(String logoFile, String text) {
        JLabel logo = getLabel(null);
        if (logoFile != null) {
            logo.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
            logo.setIcon(IconFactory.createIcon(logoFile,64));
        }
        m_licLogo.add(logo);

        JTextPane licentie = new JTextPane();
        // licentie.setFont(new Font("sans-serif", Font.PLAIN, 10));
        licentie.setForeground(Color.black);
        licentie.setBackground(getBackground());
        licentie.setEditable(false);
        licentie.setText(text);
        licentie.setAutoscrolls(true);
        licentie.setSelectionStart(0);
        licentie.setSelectionEnd(0);

        JScrollPane pane = new JScrollPane(licentie);
        pane.setMinimumSize(new Dimension(400, 100));
        pane.setPreferredSize(new Dimension(400, 100));
        m_licText.add(pane);
    }

    /**
     * adds a Contributor to an aboutbox. You need to set the textFile, logoFile
     * is optional. If you do not want to supply a textFile use addContributor
     * instead of this function.
     * 
     * @param logoFile
     *            String, the name of the file to be read as resource. This
     *            needs to be found in the classpath of the application.
     * @param textFile
     *            String, the name of the file to be read as resource. This can
     *            be a possible multi-line text with text possibly the license
     *            of the contributor
     * @throws IOException,
     *             thrown if the textFile cannot be found or accessed. remember
     *             the logoFile is optional
     */
    public void addContributorFile(String logoFile, String textFile)
            throws IOException {
        addContributor(logoFile, getTextfromFile(textFile));
    }

    public static void showDialog(Component parent) {
    	AboutPane pane = new AboutPane();
        DialogHelper.showDialog(parent,pane,"Connections");
    }

    public static void showInternalDialog(Component parent, String title, Icon icon) {
        AboutPane pane = new AboutPane();
        DialogHelper.showInternalDialog(parent,pane,title,icon);
    }
    
}
