/* 	FontChooser.java
 *	Copyright (c) 2004-2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 16-feb-2004
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
* [When]			[Who]							[What]
* Apr 3, 2004	dennis@brains2b.nl 		showDialog should set position relative to parent
* Apr 26,2005	dennis@brains2b.nl		fix for no font selected
*/
package org.brains2b.swingext.fontchooser;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GraphicsEnvironment;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.border.BevelBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.brains2b.swingext.dialog.DialogHelper;
import org.brains2b.swingext.form.Form;
import org.brains2b.swingext.form.FormAction;

/**
 * @version 	0.10.1
 * @author		Dennis Groenendijk
 */
public class FontChooserPane extends JPanel implements Form {
	
	private class SelectEventHandler implements ListSelectionListener {
		/*
		 * @see ListSelectionListener#valueChanged(ListSelectionEvent)
		 */
		public void valueChanged(ListSelectionEvent e) {
			updateExample(getSelectedFont());
		}

	}
	
	private JList listFont;
	private JList listStyle;
	private JList listSize;
	private JLabel lblExample;
	
	private Font mFont;
	
    /**
	 * Constructor for FontChooser.
	 */
	public FontChooserPane() {
		this(null);
	}
	
	public FontChooserPane(Font f) {
		super();
		mFont=f;
		init();
	}
	
	public static Font showDialog(Component parent, Font f, String title) {
		 FontChooserPane fc=new FontChooserPane(f);
		 DialogHelper.showDialog(parent, fc, title);
		 return fc.getResult();
	}
	
    public static Font showInternalDialog(Component parent, Font f, String title) {
         FontChooserPane fc=new FontChooserPane(f);
         DialogHelper.showInternalDialog(parent, fc, title, null);
         return fc.getResult();
    }
	
	
	private void init() {
		GraphicsEnvironment ge=GraphicsEnvironment.getLocalGraphicsEnvironment();
		SelectEventHandler seh=new SelectEventHandler();
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEmptyBorder(5,10,5,10));
			
		listFont=new JList();
		listFont.setListData(ge.getAvailableFontFamilyNames());
		listFont.addListSelectionListener(seh);
		JScrollPane pnlFont=new JScrollPane(listFont);
		pnlFont.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(5,5,5,5),"Font: "),BorderFactory.createBevelBorder(BevelBorder.LOWERED)));
		//pnlFont.getViewport().setView(listFont);
		
		listStyle=new JList(new String[]{"Plain","Italic","Bold","Bold Italic"});
		listStyle.addListSelectionListener(seh);
		JScrollPane pnlStyle=new JScrollPane();
		pnlStyle.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(5,5,5,5),"Style: "),BorderFactory.createBevelBorder(BevelBorder.LOWERED)));
		pnlStyle.setViewportView(listStyle);
		
		listSize=new JList(new String[] {"8","9","10","11","12","14","16","18","20","22","24","26","28","30","48","72"} );
		listSize.setFixedCellWidth(3* listSize.getFont().getSize());
		listSize.addListSelectionListener(seh);
		JScrollPane pnlSize=new JScrollPane();
		pnlSize.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(5,5,5,5),"Size: "),BorderFactory.createBevelBorder(BevelBorder.LOWERED)));
		pnlSize.setViewportView(listSize);
		
		JPanel pnlMain=new JPanel();
		pnlMain.setLayout(new BoxLayout(pnlMain,BoxLayout.X_AXIS));
		pnlMain.add(pnlFont);
		pnlMain.add(pnlStyle);
		pnlMain.add(pnlSize);
		
		JPanel pnl=new JPanel();
		lblExample=new JLabel("AbCdEfGhiJkLmNoPqRsTuVwXyZ "); //Added space because some fonts screw up the Italic angle
		lblExample.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(5,5,5,10),"Example: "),BorderFactory.createBevelBorder(BevelBorder.LOWERED)));
		lblExample.setForeground(Color.black);
		pnl.setLayout(new BorderLayout());
		pnl.add(pnlMain,BorderLayout.CENTER);		
		pnl.add(lblExample,BorderLayout.SOUTH);
		add(pnl,BorderLayout.CENTER);
		add(new FormAction(this,FormAction.BTN_OK_CANCEL+FormAction.BTN_UNDO), BorderLayout.SOUTH);
		setSelectedFont(mFont);
	}
			
	public Font getResult() {
		return mFont;
	}

	private Font getSelectedFont() {
		Font f=null;
		try {
			String name=(String) listFont.getSelectedValue();
			int style=getStyle(listStyle.getSelectedIndex());
			String s_size=(String)listSize.getSelectedValue();
			if (s_size==null) s_size="10";
			int size=Integer.parseInt(s_size);
		
			f=new Font(name,style,size);
		} catch (Exception e) {
			
		}
		
		if (f==null) {
			f=mFont;
		}
			
		return f;
	}	
	
	private void setSelectedFont(Font f) {
		if (f==null) {
			listFont.setSelectedIndex(0);
			listStyle.setSelectedIndex(0);
			listSize.setSelectedIndex(0);
		} else {
			/* Unfortunatly the scrollable boolean does not work
			 * in jdk1.3.x, but is reported fixed in jdk1.4
			 * See bug #4337516 in the java bug database
			 */		
			listFont.setSelectedValue(f.getName(),true);
			listStyle.setSelectedIndex(getIndex(f.getStyle()));
			listSize.setSelectedValue(String.valueOf(f.getSize()),true);
		}
		updateExample(f);
	}
	

	private int getStyle(int index) {
		switch (index) {
			case 0:
				return Font.PLAIN;
			case 1:
				return Font.ITALIC;
			case 2:
				return Font.BOLD;
			case 3:
				return Font.ITALIC+Font.BOLD;
			default:
				return 0;
		}		
		
	}
	
	private int getIndex(int style) {
		switch (style) {
			case Font.PLAIN:
				return 0;
			case Font.ITALIC:
				return 1;
			case Font.BOLD:
				return 2;
			case Font.ITALIC+Font.BOLD:
				return 3;
			default:
				return 0;
		}
	}

	private void updateExample(Font f) {
		lblExample.setFont(f);
		lblExample.repaint();
	}		
	/*
	 * @see Form#performApply()
	 */
	public void performApply() {
		//Button not shown so never called
		performOk();
	}

	/*
	 * @see Form#performOk()
	 */
	public void performOk() {
		mFont=getSelectedFont();
		close();
	}

	/*
	 * @see Form#performUndo()
	 */
	public void performUndo() {
		setSelectedFont(mFont);
			
	}

	/*
	 * @see Form#performCancel()
	 */
	public void performCancel() {
		mFont=null;
		close();
	}

	private void close() {
		if (getRootPane().getParent() instanceof JInternalFrame) {
            ((JInternalFrame) getRootPane().getParent()).doDefaultCloseAction();      
        } else {
            SwingUtilities.getWindowAncestor(this).dispose();
        }
	}
	
}
